import { of } from 'rxjs';
import { catchError, filter, map } from 'rxjs/operators';
import { HttpClient, HttpEventType, HttpRequest } from '@angular/common/http';
import { InjectionToken, Injectable, NgModule } from '@angular/core';
import { NtFormFieldControl } from '@ng-tangram/components/forms';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
const NtUploadStatus = {
    BEGIN: 0,
    SENDING: 1,
    SUCCESS: 2,
    ERROR: 3,
    UPLOADED: 4,
};
NtUploadStatus[NtUploadStatus.BEGIN] = 'BEGIN';
NtUploadStatus[NtUploadStatus.SENDING] = 'SENDING';
NtUploadStatus[NtUploadStatus.SUCCESS] = 'SUCCESS';
NtUploadStatus[NtUploadStatus.ERROR] = 'ERROR';
NtUploadStatus[NtUploadStatus.UPLOADED] = 'UPLOADED';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtFormDataUploadHandler {
    /**
     * @param {?} _http
     */
    constructor(_http) {
        this._http = _http;
    }
    /**
     * @template T
     * @param {?} url
     * @param {?} file
     * @param {?} filename
     * @param {?=} handler
     * @return {?}
     */
    upload(url, file, filename, handler = {}) {
        /** @type {?} */
        const formData = this._getFormData(file, filename);
        return this._http.request(new HttpRequest('POST', url, formData, { reportProgress: true }))
            .pipe(map((/**
         * @param {?} event
         * @return {?}
         */
        event => this._progressHandler(event, handler))), filter((/**
         * @param {?} event
         * @return {?}
         */
        event => event.type === HttpEventType.Response)), map((/**
         * @param {?} event
         * @return {?}
         */
        (event) => {
            return {
                status: event.status >= 200 && event.status < 400
                    ? NtUploadStatus.SUCCESS
                    : NtUploadStatus.ERROR,
                data: event.body
            };
        })), catchError((/**
         * @param {?} error
         * @return {?}
         */
        error => of({
            status: NtUploadStatus.ERROR,
            error: error.statusText
        }))));
    }
    /**
     * @private
     * @param {?} event
     * @param {?=} handler
     * @return {?}
     */
    _progressHandler(event, handler = {}) {
        if (event.type === HttpEventType.Sent && handler.begin) {
            handler.begin(event);
        }
        else if (event.type === HttpEventType.UploadProgress && handler.progress) {
            handler.progress(event);
        }
        else if (event.type === HttpEventType.Response && handler.done) {
            handler.done();
        }
        return event;
    }
    /**
     * @private
     * @param {?} file
     * @param {?} filename
     * @return {?}
     */
    _getFormData(file, filename) {
        if (filename) {
            /** @type {?} */
            const formData = new FormData();
            formData.append(filename, file);
            return formData;
        }
        return file;
    }
}
NtFormDataUploadHandler.decorators = [
    { type: Injectable },
];
/** @nocollapse */
NtFormDataUploadHandler.ctorParameters = () => [
    { type: HttpClient }
];
/** @type {?} */
const NT_UPLOAD_HANDLER = new InjectionToken('nt-upload-handler');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtUploadModule {
    /**
     * @param {?=} handler
     * @return {?}
     */
    static forRoot(handler) {
        return {
            ngModule: NtUploadModule,
            providers: [
                { provide: NT_UPLOAD_HANDLER, useClass: handler || NtFormDataUploadHandler }
            ]
        };
    }
}
NtUploadModule.decorators = [
    { type: NgModule },
];

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
class NtUploadFile {
    /**
     * @param {?} name
     * @param {?=} size
     * @param {?=} type
     * @param {?=} link
     */
    constructor(name, size, type, link) {
        this.name = name;
        this.size = size;
        this.type = type;
        this.link = link;
        this.status = NtUploadStatus.BEGIN;
    }
}
/**
 * @abstract
 * @template T
 */
class NtUploadControl extends NtFormFieldControl {
    /**
     * @param {?} _uploader
     * @param {?} ngControl
     */
    constructor(_uploader, ngControl) {
        super();
        this._uploader = _uploader;
        this.ngControl = ngControl;
        this._onChange = (/**
         * @return {?}
         */
        () => { });
        this._onTouched = (/**
         * @return {?}
         */
        () => { });
        if (this.ngControl) {
            this.ngControl.valueAccessor = this;
        }
    }
    /**
     * @param {?} value
     * @return {?}
     */
    writeValue(value) {
        if (value) {
            this.setValue(value);
        }
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnChange(fn) {
        this._onChange = fn;
    }
    /**
     * @param {?} fn
     * @return {?}
     */
    registerOnTouched(fn) {
        this._onTouched = fn;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
class NtFileAcceptError {
    /**
     * @param {?} file
     * @param {?} fileAccept
     * @param {?=} allowAccepts
     */
    constructor(file, fileAccept, allowAccepts) {
        this.file = file;
        this.fileAccept = fileAccept;
        this.allowAccepts = allowAccepts;
    }
}
class NtFileSizeError {
    /**
     * @param {?} file
     * @param {?} maxSize
     * @param {?=} maxSizeString
     */
    constructor(file, maxSize, maxSizeString) {
        this.file = file;
        this.maxSize = maxSize;
        this.maxSizeString = maxSizeString;
    }
}
class NtFileUploadError {
    /**
     * @param {?} status
     * @param {?} statusText
     */
    constructor(status, statusText) {
        this.status = status;
        this.statusText = statusText;
    }
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NtFormDataUploadHandler, NT_UPLOAD_HANDLER, NtUploadModule, NtUploadFile, NtUploadControl, NtUploadStatus, NtFileAcceptError, NtFileSizeError, NtFileUploadError };
//# sourceMappingURL=upload.js.map
