import { of } from 'rxjs';
import { catchError, filter, map } from 'rxjs/operators';
import { HttpClient, HttpEventType, HttpRequest } from '@angular/common/http';
import { InjectionToken, Injectable, NgModule } from '@angular/core';
import { NtFormFieldControl } from '@ng-tangram/components/forms';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/** @enum {number} */
var NtUploadStatus = {
    BEGIN: 0,
    SENDING: 1,
    SUCCESS: 2,
    ERROR: 3,
    UPLOADED: 4,
};
NtUploadStatus[NtUploadStatus.BEGIN] = 'BEGIN';
NtUploadStatus[NtUploadStatus.SENDING] = 'SENDING';
NtUploadStatus[NtUploadStatus.SUCCESS] = 'SUCCESS';
NtUploadStatus[NtUploadStatus.ERROR] = 'ERROR';
NtUploadStatus[NtUploadStatus.UPLOADED] = 'UPLOADED';

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NtFormDataUploadHandler = /** @class */ (function () {
    function NtFormDataUploadHandler(_http) {
        this._http = _http;
    }
    /**
     * @template T
     * @param {?} url
     * @param {?} file
     * @param {?} filename
     * @param {?=} handler
     * @return {?}
     */
    NtFormDataUploadHandler.prototype.upload = /**
     * @template T
     * @param {?} url
     * @param {?} file
     * @param {?} filename
     * @param {?=} handler
     * @return {?}
     */
    function (url, file, filename, handler) {
        var _this = this;
        if (handler === void 0) { handler = {}; }
        /** @type {?} */
        var formData = this._getFormData(file, filename);
        return this._http.request(new HttpRequest('POST', url, formData, { reportProgress: true }))
            .pipe(map((/**
         * @param {?} event
         * @return {?}
         */
        function (event) { return _this._progressHandler(event, handler); })), filter((/**
         * @param {?} event
         * @return {?}
         */
        function (event) { return event.type === HttpEventType.Response; })), map((/**
         * @param {?} event
         * @return {?}
         */
        function (event) {
            return {
                status: event.status >= 200 && event.status < 400
                    ? NtUploadStatus.SUCCESS
                    : NtUploadStatus.ERROR,
                data: event.body
            };
        })), catchError((/**
         * @param {?} error
         * @return {?}
         */
        function (error) { return of({
            status: NtUploadStatus.ERROR,
            error: error.statusText
        }); })));
    };
    /**
     * @private
     * @param {?} event
     * @param {?=} handler
     * @return {?}
     */
    NtFormDataUploadHandler.prototype._progressHandler = /**
     * @private
     * @param {?} event
     * @param {?=} handler
     * @return {?}
     */
    function (event, handler) {
        if (handler === void 0) { handler = {}; }
        if (event.type === HttpEventType.Sent && handler.begin) {
            handler.begin(event);
        }
        else if (event.type === HttpEventType.UploadProgress && handler.progress) {
            handler.progress(event);
        }
        else if (event.type === HttpEventType.Response && handler.done) {
            handler.done();
        }
        return event;
    };
    /**
     * @private
     * @param {?} file
     * @param {?} filename
     * @return {?}
     */
    NtFormDataUploadHandler.prototype._getFormData = /**
     * @private
     * @param {?} file
     * @param {?} filename
     * @return {?}
     */
    function (file, filename) {
        if (filename) {
            /** @type {?} */
            var formData = new FormData();
            formData.append(filename, file);
            return formData;
        }
        return file;
    };
    NtFormDataUploadHandler.decorators = [
        { type: Injectable },
    ];
    /** @nocollapse */
    NtFormDataUploadHandler.ctorParameters = function () { return [
        { type: HttpClient }
    ]; };
    return NtFormDataUploadHandler;
}());
/** @type {?} */
var NT_UPLOAD_HANDLER = new InjectionToken('nt-upload-handler');

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NtUploadModule = /** @class */ (function () {
    function NtUploadModule() {
    }
    /**
     * @param {?=} handler
     * @return {?}
     */
    NtUploadModule.forRoot = /**
     * @param {?=} handler
     * @return {?}
     */
    function (handler) {
        return {
            ngModule: NtUploadModule,
            providers: [
                { provide: NT_UPLOAD_HANDLER, useClass: handler || NtFormDataUploadHandler }
            ]
        };
    };
    NtUploadModule.decorators = [
        { type: NgModule },
    ];
    return NtUploadModule;
}());

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
/**
 * @template T
 */
var  /**
 * @template T
 */
NtUploadFile = /** @class */ (function () {
    function NtUploadFile(name, size, type, link) {
        this.name = name;
        this.size = size;
        this.type = type;
        this.link = link;
        this.status = NtUploadStatus.BEGIN;
    }
    return NtUploadFile;
}());
/**
 * @abstract
 * @template T
 */
var  /**
 * @abstract
 * @template T
 */
NtUploadControl = /** @class */ (function (_super) {
    __extends(NtUploadControl, _super);
    function NtUploadControl(_uploader, ngControl) {
        var _this = _super.call(this) || this;
        _this._uploader = _uploader;
        _this.ngControl = ngControl;
        _this._onChange = (/**
         * @return {?}
         */
        function () { });
        _this._onTouched = (/**
         * @return {?}
         */
        function () { });
        if (_this.ngControl) {
            _this.ngControl.valueAccessor = _this;
        }
        return _this;
    }
    /**
     * @param {?} value
     * @return {?}
     */
    NtUploadControl.prototype.writeValue = /**
     * @param {?} value
     * @return {?}
     */
    function (value) {
        if (value) {
            this.setValue(value);
        }
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    NtUploadControl.prototype.registerOnChange = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this._onChange = fn;
    };
    /**
     * @param {?} fn
     * @return {?}
     */
    NtUploadControl.prototype.registerOnTouched = /**
     * @param {?} fn
     * @return {?}
     */
    function (fn) {
        this._onTouched = fn;
    };
    return NtUploadControl;
}(NtFormFieldControl));

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */
var NtFileAcceptError = /** @class */ (function () {
    function NtFileAcceptError(file, fileAccept, allowAccepts) {
        this.file = file;
        this.fileAccept = fileAccept;
        this.allowAccepts = allowAccepts;
    }
    return NtFileAcceptError;
}());
var NtFileSizeError = /** @class */ (function () {
    function NtFileSizeError(file, maxSize, maxSizeString) {
        this.file = file;
        this.maxSize = maxSize;
        this.maxSizeString = maxSizeString;
    }
    return NtFileSizeError;
}());
var NtFileUploadError = /** @class */ (function () {
    function NtFileUploadError(status, statusText) {
        this.status = status;
        this.statusText = statusText;
    }
    return NtFileUploadError;
}());

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

/**
 * @fileoverview added by tsickle
 * @suppress {checkTypes,extraRequire,missingOverride,missingReturn,unusedPrivateMembers,uselessCode} checked by tsc
 */

export { NtFormDataUploadHandler, NT_UPLOAD_HANDLER, NtUploadModule, NtUploadFile, NtUploadControl, NtUploadStatus, NtFileAcceptError, NtFileSizeError, NtFileUploadError };
//# sourceMappingURL=upload.js.map
